use anyhow::{anyhow, Context};
use clap::Parser;
use rayon::prelude::*;
use std::ffi::OsString;
use std::fs;
use std::os::unix::prelude::OsStringExt;
use std::path::PathBuf;
use std::process::Command;

#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
struct Platform<'a> {
    arch: &'a str,
    os_id: &'a str,
    compiler: &'a str,
    opt_level: &'a str,
}

impl<'a> TryFrom<&'a str> for Platform<'a> {
    type Error = anyhow::Error;
    fn try_from(value: &'a str) -> Result<Self, Self::Error> {
        let chunks: Vec<&str> = value.split('-').collect();
        if chunks.len() != 4 {
            Err(anyhow!("invalid platform"))
        } else {
            Ok(Platform {
                arch: chunks[0],
                os_id: chunks[1],
                compiler: chunks[2],
                opt_level: chunks[3],
            })
        }
    }
}

impl<'a> ToString for Platform<'a> {
    fn to_string(&self) -> String {
        format!(
            "{}-{}-{}-{}",
            self.arch, self.os_id, self.compiler, self.opt_level
        )
    }
}

fn get_apptainer_root<'a>(platform: &Platform<'a>) -> Option<PathBuf> {
    let first_chunk =
        |value: &'a str| -> &'a str { value.split_once('+').unwrap_or((value, "")).0 };
    let arch = if platform.arch.starts_with("x86_64_") {
        "x86_64"
    } else if platform.arch.starts_with("armv") {
        "aarch64"
    } else {
        first_chunk(platform.arch)
    };
    let os_id = if platform.os_id.starts_with("centos9") {
        "el9"
    } else {
        first_chunk(platform.os_id)
    };
    match (arch, os_id) {
        ("x86_64", "centos7") => Some("/cvmfs/cernvm-prod.cern.ch/cvm4".into()),
        ("x86_64", "el9") => {
            Some("/cvmfs/lhcb.cern.ch/containers/os-base/alma9-devel/prod/amd64".into())
        }
        _ => None,
    }
}

fn check(toolchain: impl Into<PathBuf>, project_dir: impl Into<PathBuf>) -> anyhow::Result<()> {
    let toolchain: PathBuf =
        std::fs::canonicalize(toolchain.into()).context("cannot find toolchain")?;
    let platform: Platform = toolchain
        .file_stem()
        .and_then(|x| x.to_str())
        .map_or(Err(anyhow!("invalid toolchain path")), |x| x.try_into())?;
    let mut approot = get_apptainer_root(&platform).ok_or(anyhow!("unsupported platform"))?;
    approot = std::fs::canonicalize(approot).context("cannot find apptainer root")?;

    let project_dir: PathBuf =
        std::fs::canonicalize(project_dir.into()).context("cannot find project dir")?;
    let build_dir = project_dir.join(format!("build.{}", platform.to_string()));

    // println!(
    //     "{} -> {} -> {}",
    //     toolchain.display(),
    //     platform.to_string(),
    //     approot.display(),
    // );
    let mut toolchain_opt = OsString::from("-DCMAKE_TOOLCHAIN_FILE=");
    toolchain_opt.push(toolchain);

    if build_dir.exists() {
        fs::remove_dir_all(&build_dir)?;
    }
    fs::create_dir(&build_dir)?;

    let mut cmd = Command::new("apptainer");
    cmd.args(["exec", "-B", "/cvmfs"]);

    if let Some(path) = std::env::var_os("PATH") {
        let mut path_opt = OsString::from("PATH=");
        path_opt.push(path);
        cmd.arg("--env");
        cmd.arg(path_opt);
    }

    cmd.arg(approot)
        .args(["cmake", "-S"])
        .arg(project_dir)
        .arg("-B")
        .arg(build_dir)
        .arg(toolchain_opt);
    // println!("{:?}", cmd);
    let proc = cmd.output().context("failed to run apptainer")?;
    if proc.status.success() {
        Ok(())
    } else {
        println!("{}", OsString::from_vec(proc.stderr).to_str().unwrap());
        Err(anyhow!("failed execution"))
    }
}

#[derive(Parser, Debug)]
struct Args {
    project: PathBuf,
    #[arg(required = true)]
    toolchains: Vec<PathBuf>,
}

fn main() -> Result<(), &'static str> {
    let args = Args::parse();

    let failures: Vec<_> = args
        .toolchains
        .par_iter()
        .map(|toolchain| (toolchain, check(toolchain, &args.project)))
        .filter_map(|x| {
            if let Err(problem) = x.1 {
                Some((x.0, problem))
            } else {
                None
            }
        })
        .collect();

    if !failures.is_empty() {
        println!("failed toolchains:");
        for (toolchain, problem) in failures {
            println!("- {}: {}", toolchain.display(), problem);
        }
        return Err("some toolchains failed");
    }
    Ok(())
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn check_platform() {
        // good parse
        assert_eq!(
            Platform::try_from("a-b-c-d").expect("failed to parse a-b-c-d"),
            Platform {
                arch: "a",
                os_id: "b",
                compiler: "c",
                opt_level: "d"
            },
        );
        // invalid platform strings
        for bad_platform in ["", "foo", "foo-bar", "foo-bar-baz", "a-b-c-d-e"] {
            assert!(
                Platform::try_from(bad_platform).is_err(),
                "platform string '{}' should not parse",
                bad_platform
            );
        }
        // to_string
        assert_eq!(
            Platform {
                arch: "a",
                os_id: "b",
                compiler: "c",
                opt_level: "d"
            }
            .to_string(),
            "a-b-c-d"
        );
        // roundtrip
        assert_eq!(
            Platform::try_from("1-2-3-4").unwrap().to_string(),
            "1-2-3-4"
        );
    }
}
