#!/usr/bin/env python3
###############################################################################
# (c) Copyright 2023 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
from multiprocessing import Pool
from os import environ
from pathlib import Path
from shutil import rmtree
from subprocess import run
from typing import Optional, Union

APPTAINER_ROOTS = {
    "x86_64-centos7": "/cvmfs/cernvm-prod.cern.ch/cvm4",
    "x86_64-el9": "/cvmfs/lhcb.cern.ch/containers/os-base/alma9-devel/prod/amd64",
}


def get_apptainer_root(platform: str) -> Optional[Path]:
    arch, os_id, _ = platform.split("-", 2)
    if arch.startswith("x86_64_"):
        arch = "x86_64"
    elif arch.startswith("armv8"):
        arch = "aarch64"
    os_id = os_id.split("+", 1)[0]
    if os_id == "centos9":
        os_id = "el9"
    return APPTAINER_ROOTS.get(f"{arch}-{os_id}")


def check(toolchain: Union[Path, str], approot: Optional[Path] = None) -> bool:
    if isinstance(toolchain, str):
        toolchain = Path(toolchain)

    assert toolchain.exists()
    if approot is None:
        approot = get_apptainer_root(toolchain.stem)
    assert approot, toolchain

    source_dir = Path(__file__).parent / "project"
    build_dir = source_dir / f"build.{toolchain.stem}"
    if build_dir.exists():
        rmtree(build_dir)
    build_dir.mkdir()
    result = run(
        [
            "apptainer",
            "exec",
            "-B",
            "/cvmfs",
            "--env",
            f"PATH={environ['PATH']}",
            str(approot.absolute()),
            "cmake",
            "-S",
            str(source_dir),
            "-B",
            str(build_dir),
            f"-DCMAKE_TOOLCHAIN_FILE={toolchain.absolute()}",
        ],
        capture_output=True,
    )
    return result.returncode == 0


def main():
    from argparse import ArgumentParser

    parser = ArgumentParser()
    parser.add_argument("toolchain", nargs="+", help="toolchains to test", type=Path)
    parser.add_argument(
        "-n", "--dry-run", action="store_true", help="do not run cmake, just list"
    )

    args = parser.parse_args()

    testable: list[(Path, Path)] = []
    skipped: list[Path] = []
    for toolchain in args.toolchain:
        approot = get_apptainer_root(toolchain.stem)
        if approot is None:
            skipped.append(toolchain)
        else:
            testable.append((toolchain, approot))
    if skipped:
        print("ignoring:\n-", "\n- ".join(map(str, skipped)))

    if args.dry_run:
        print("would test:")
        for toolchain, approot in testable:
            print(f"- {toolchain} via {approot}")
        return

    failed = []
    with Pool() as pool:
        for (toolchain, _), success in zip(testable, pool.starmap(check, testable)):
            if not success:
                failed.append(toolchain)
    if failed:
        print("failed:\n-", "\n- ".join(map(str, failed)))
        exit(1)


if __name__ == "__main__":
    main()
